#!/usr/bin/env python

#-----------------------------------------------------------------------
# lockinuser.py
# Author: Bob Dondero
#-----------------------------------------------------------------------

import threading

#-----------------------------------------------------------------------

class BankAcct:

    def __init__(self):
        self._balance = 0

    def get_balance(self):
        return self._balance

    def deposit(self, amount):
        temp = self._balance
        temp += amount
        print(temp)
        self._balance = temp

    def withdraw(self, amount):
        temp = self._balance
        temp -= amount
        print(temp)
        self._balance = temp

#-----------------------------------------------------------------------

class DepositThread (threading.Thread):

    def __init__(self, bank_acct, bank_acct_lock):
        threading.Thread.__init__(self)
        self._bank_acct = bank_acct
        self._bank_acct_lock = bank_acct_lock

    def run(self):
        for _ in range(10):
            self._bank_acct_lock.acquire()
            self._bank_acct.deposit(1)
            self._bank_acct_lock.release()

#-----------------------------------------------------------------------

class WithdrawThread (threading.Thread):

    def __init__(self, bank_acct, bank_acct_lock):
        threading.Thread.__init__(self)
        self._bank_acct = bank_acct
        self._bank_acct_lock = bank_acct_lock

    def run(self):
        for _ in range(5):
            self._bank_acct_lock.acquire()
            self._bank_acct.withdraw(2)
            self._bank_acct_lock.release()

#-----------------------------------------------------------------------

def main():

    bank_acct = BankAcct()
    bank_acct_lock = threading.RLock()

    deposit_thread = DepositThread(bank_acct, bank_acct_lock)
    withdraw_thread = WithdrawThread(bank_acct, bank_acct_lock)

    deposit_thread.start()
    withdraw_thread.start()

    deposit_thread.join()
    withdraw_thread.join()

    print('Final balance:', bank_acct.get_balance())

#-----------------------------------------------------------------------

if __name__ == '__main__':
    main()
